/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro
 Maria del Carmen Sánchez Medrano

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
 package org.javafmi.kernel.v2.nativeaccess;

import com.sun.jna.Memory;
import com.sun.jna.Native;
import com.sun.jna.Pointer;
import org.javafmi.exceptions.MethodExecutionException;

import java.util.List;

public class AllocateMemory implements NativeInterface.AllocateMemory {
    private static final int MEMORY_OFFSET = 4;
    private List<Pointer> onMemoryAllocatedObjects;

    public AllocateMemory(List<Pointer> onMemoryAllocatedObjects) {
        this.onMemoryAllocatedObjects = onMemoryAllocatedObjects;
    }

    @Override
    public Pointer allocate(int numberOfObjects, int size) {
        long peer = Native.malloc(numberOfObjects * size);
        if(peer <= 0) throw new MethodExecutionException("Impossible to allocate space");
        Pointer pointer = new Pointer(peer);
        pointer.clear(numberOfObjects * size);
        onMemoryAllocatedObjects.add(pointer);
        return pointer;
    }
}
