/*

 Copyright 2013, 2014 SIANI - ULPGC
 Jose Juan Hernandez Cabrera
 Jose Evora Gomez
 Johan Sebastian Cortes Montenegro

 This File is Part of JavaFMI Project

 JavaFMI Project is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 JavaFMI Project is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with JavaFMI Library.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.javafmi.kernel.v2rc2.modeldescription;

import org.javafmi.exceptions.NoSuchVariableException;
import org.simpleframework.xml.Attribute;
import org.simpleframework.xml.Element;
import org.simpleframework.xml.ElementList;
import org.simpleframework.xml.Root;

import java.util.*;

@Root(name = "fmiModelDescription")
public class ModelDescription implements org.javafmi.kernel.modeldescription.ModelDescription {

    @Attribute(required = true)
    private String fmiVersion;
    @Attribute(required = true)
    private String modelName;
    @Attribute(required = true)
    private String guid;
    @Attribute(required = false)
    private String description;
    @Attribute(required = false)
    private String author;
    @Attribute(required = false)
    private String version;
    @Attribute(required = false)
    private String copyright;
    @Attribute(required = false)
    private String licence;
    @Attribute(required = false)
    private String generationTool;
    @Attribute(required = false)
    private String generationDateAndTime;
    @Attribute(required = false)
    private String variableNamingConvention;
    @Attribute(required = false)
    private int numberOfEventIndicators;

    @ElementList(name = "UnitDefinitions", required = false)
    private List<Unit> unitDefinitions;
    @ElementList(name = "TypeDefinitions", required = false)
    private List<SimpleType> typeDefinitions;
    @ElementList(name = "LogCategories", required = false)
    private List<Category> logCategories;
    @Element(name = "DefaultExperiment", required = false)
    private DefaultExperiment defaultExperiment;
    @ElementList(name = "VendorAnnotations", required = false)
    private List<Tool> vendorAnnotations;
    @ElementList(name = "ModelVariables", required = true)
    private List<ScalarVariable> modelVariablesList;
    @Element(name = "ModelStructure", required = true)
    private ModelStructure modelStructure;
    @Element(name = "ModelExchange", required = false)
    private ModelExchange modelExchange;
    @Element(name = "CoSimulation", required = false)
    private CoSimulation coSimulation;

    private Map<String, ScalarVariable> modelVariables;
    private Map<Integer,String> nameReferences;

    public String getNameReferences(int vr){
        return this.nameReferences.get(vr);
    }
    public String getModelIdentifier() {
        return coSimulation.getModelIdentifier();
    }

    public org.javafmi.kernel.modeldescription.ScalarVariable[] getModelVariables() {
        return modelVariablesList.toArray(new org.javafmi.kernel.modeldescription.ScalarVariable[modelVariablesList.size()]);
    }

    public SimpleType getTypeFromTypeDefinition(String name) {
        for (SimpleType type : typeDefinitions)
            if (type.getName().equals(name)) return type.getType();
        return null;
    }

    public ScalarVariable getScalarVariable(String variableName) {
        ScalarVariable scalarVariable = modelVariables.get(variableName);
        if (scalarVariable == null) throw new NoSuchVariableException(variableName);
        else return scalarVariable;
    }

    public String getModelName() {
        return modelName;
    }

    public String getGuid() {
        return guid;
    }

    public CoSimulation getCoSimulation() {
        return coSimulation;
    }

    public void buildAttributes() {
        modelVariables = new LinkedHashMap<>();
        for (ScalarVariable scalarVariable : modelVariablesList) {
            scalarVariable.buildAttributes();
            modelVariables.put(scalarVariable.getName(), scalarVariable);
        }
    }

    public void addVariable(ScalarVariable variable) {
        variable.setValueReference(this.nameReferences.size());
        this.nameReferences.put(this.nameReferences.size(),variable.getName());
        this.modelVariablesList.add(variable);
        this.modelVariables.put(variable.getName(), variable);

    }

    public boolean canGetAndSetFmuState() {
        return coSimulation.canGetAndSetFmuState();
    }

    public boolean canSerializeFMUState() {
        return coSimulation.canSerializeFMUState();
    }
}
